<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:deltaxml="http://www.deltaxml.com/ns/well-formed-delta-v1"
  xmlns:saxon="http://saxon.sf.net/" exclude-result-prefixes="xs" version="2.0">

  <xsl:include href="functions/merge-functions.xsl"/>

  <xsl:param as="xs:string" name="base-override" select="''"/>

  <xsl:template match="@* | node()">
    <xsl:copy>
      <xsl:apply-templates select="@*, node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="deltaxml:textGroup">
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <xsl:variable name="editType" select="deltaxml:leafEditType(.)"/>
      <xsl:if test="$editType = ('add', 'delete', 'modify')">
        <xsl:attribute name="deltaxml:edit-type" select="$editType"/>
      </xsl:if>
      <xsl:apply-templates select="deltaxml:text">
        <!-- we'll sort the text children by contribution size, doesn't affect sementics, but useful
              in the following defaults filter -->
        <xsl:sort select="count(tokenize(@deltaxml:deltaV2, '='))" order="descending"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="/*//*[not(ancestor-or-self::deltaxml:*)][@deltaxml:deltaV2]">
    <xsl:copy>
      <xsl:if test="deltaxml:elementEditType(.) = ('add', 'delete', 'modify')">
        <xsl:attribute name="deltaxml:edit-type" select="deltaxml:elementEditType(.)"/>
      </xsl:if>
      <xsl:apply-templates select="@*, node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="deltaxml:attributes/*">
    <xsl:copy>
      <xsl:if test="deltaxml:leafEditType(.) = ('add', 'delete', 'modify')">
        <xsl:attribute name="deltaxml:edit-type" select="deltaxml:leafEditType(.)"/>
      </xsl:if>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="deltaxml:attributeValue">
        <!-- we'll sort the attributeValue children by contribution size, doesn't affect sementics, but useful
           in the following defaults filter -->
        <xsl:sort select="count(tokenize(@deltaxml:deltaV2, '='))" order="descending"/>
      </xsl:apply-templates>

    </xsl:copy>
  </xsl:template>



  <xsl:function name="deltaxml:elementEditType" as="xs:string">
    <xsl:param name="currentElement" as="element()"/>
    <xsl:variable name="childVersions" as="xs:string+"
      select="deltaxml:getVersions($currentElement/@deltaxml:deltaV2)"/>
    <xsl:variable name="parentVersions" as="xs:string+"
      select="deltaxml:getParentVersions($currentElement)"/>
    <xsl:variable name="baseVersion"
      select="if ($base-override ne '') then $base-override else deltaxml:getBase($currentElement)"
      as="xs:string"/>
    <xsl:variable name="isDelete" as="xs:boolean"
      select="($childVersions = $baseVersion) and (count($parentVersions) gt count($childVersions))"/>
    <xsl:variable name="isAdd" as="xs:boolean"
      select="($parentVersions = $baseVersion) and (not($childVersions = $baseVersion))"/>
    <xsl:choose>
      <xsl:when test="$isAdd">
        <xsl:text>add</xsl:text>
      </xsl:when>
      <xsl:when test="$isDelete">
        <xsl:text>delete</xsl:text>
      </xsl:when>
      <xsl:when test="count($parentVersions) gt count($childVersions)">
        <xsl:text>modify</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>other</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>


  <xsl:function name="deltaxml:leafEditType" as="xs:string">
    <xsl:param name="currentElement" as="element()"/>
    <xsl:variable name="childVersions" as="xs:string+"
      select="deltaxml:getVersions($currentElement/@deltaxml:deltaV2)"/>
    <xsl:variable name="parentVersions" as="xs:string+"
      select="deltaxml:getParentVersions($currentElement)"/>
    <xsl:variable name="baseVersion"
      select="if ($base-override ne '') then $base-override else deltaxml:getBase($currentElement)"
      as="xs:string"/>
    <xsl:variable name="isDelete" as="xs:boolean"
      select="($childVersions = $baseVersion) and (count($parentVersions) gt count($childVersions))"/>
    <xsl:variable name="isAdd" as="xs:boolean"
      select="($parentVersions = $baseVersion) and (not($childVersions = $baseVersion))"/>
    <xsl:variable name="isModify" as="xs:boolean"
      select="contains($currentElement/@deltaxml:deltaV2, '!=')"/>
    <xsl:variable name="isCompound" as="xs:boolean"
      select="not($parentVersions = $baseVersion) and (count($parentVersions) gt count($childVersions))"/>
    <xsl:choose>
      <xsl:when test="$isAdd">
        <xsl:text>add</xsl:text>
      </xsl:when>
      <xsl:when test="$isDelete">
        <xsl:text>delete</xsl:text>
      </xsl:when>
      <xsl:when test="$isModify">
        <xsl:text>modify</xsl:text>
      </xsl:when>
      <xsl:when test="$isCompound">
        <xsl:text>modify</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>unchanged</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
</xsl:stylesheet>
