<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:deltaxml="http://www.deltaxml.com/ns/well-formed-delta-v1"
  xmlns:saxon="http://saxon.sf.net/"
  exclude-result-prefixes="xs"
  version="2.0">

  <xsl:include href="functions/merge-functions.xsl"/>
 
  <xsl:template match="@* | node()">
    <xsl:copy>
      <xsl:apply-templates select="@*, node()"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="deltaxml:text | deltaxml:attributeValue">
    <xsl:variable name="isChosen" as="xs:boolean">
      <xsl:variable name="baseVersion" select="deltaxml:getBase(.)" as="xs:string"/>
      <xsl:choose>
        <xsl:when test="(parent::*[@deltaxml:edit-type='add']) and (count(preceding-sibling::*) eq 0)">
          <xsl:sequence select="true()"/>
        </xsl:when>
        <xsl:when test="parent::*[@deltaxml:edit-type=('modify', 'delete')]">
          <xsl:variable name="baseNotIncludedInAnyLeaves" as="xs:boolean"
            select="empty(parent::*[deltaxml:deltaV2Includes(@deltaxml:deltaV2, $baseVersion)])"/>
          <xsl:variable name="isFirstLeaf" as="xs:boolean" select="count(preceding-sibling::*) eq 0"/>
          <xsl:variable name="currentLeafIncludesBase" as="xs:boolean" select="deltaxml:deltaV2Includes(@deltaxml:deltaV2, $baseVersion)"/>
          <xsl:choose>
            <!-- if there is a leaf which contains the base version, then it is 'chosen'
              Otherwise we'll take the first leaf if none contain the ancestor -->
            <xsl:when test="$baseNotIncludedInAnyLeaves and $isFirstLeaf">
              <xsl:sequence select="true()"/>
            </xsl:when>
            <xsl:when test="$currentLeafIncludesBase">
              <xsl:sequence  select="true()"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:sequence select="false()"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:copy>
      <xsl:attribute name="deltaxml:chosen" select="$isChosen"/>
      <xsl:apply-templates select="@*, node()"/>
    </xsl:copy>
  </xsl:template>
  
  
  
   <xsl:template match="*[not(ancestor-or-self::deltaxml:*)][@deltaxml:edit-type]">
    <xsl:copy>
    <xsl:variable name="isChosen" as="xs:boolean" select="@deltaxml:edit-type =('add', 'modify', 'delete')"/>
      <xsl:attribute name="deltaxml:chosen" select="$isChosen" />
      <xsl:apply-templates select="@*, node()"/>
    </xsl:copy>
  </xsl:template>
  
  
</xsl:stylesheet>