<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:deltaxml="http://www.deltaxml.com/ns/well-formed-delta-v1"
                xmlns:dxml-merge="http://www.deltaxml.com/ns/dxml-merge"
                xmlns:dxml-merge-group="http://www.deltaxml.com/ns/dxml-merge-group"
                version="2.0"
                exclude-result-prefixes="xs xsl">
  
  <xsl:include href="functions/merge-functions.xsl"/>

  <xsl:template match="@* | node()">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="@DITAArchVersion | @version | @content-type | @deltaxml:deltaV2"/>
  
  <xsl:template match="deltaxml:textGroup[not(contains(@deltaxml:deltaV2, '!=')) and
                                          count(tokenize(@deltaxml:deltaV2, '!=|='))=count(tokenize(ancestor::*[@deltaxml:deltaV2][1]/@deltaxml:deltaV2, '!=|='))]">
    <xsl:value-of select="deltaxml:text/text()"/>
  </xsl:template>
  
  <xsl:template match="deltaxml:textGroup[@deltaxml:edit-type]">
    <xsl:variable name="textGroup-id" select="generate-id()" as="xs:string"/>
    <xsl:variable name="editType" as="xs:string" select="@deltaxml:edit-type"/>

    <dxml-merge-group:text>
      <xsl:attribute name="group" select="$textGroup-id"/>
      <xsl:attribute name="edit-type" select="$editType"/>
      <xsl:attribute name="deltaV2" select="@deltaxml:deltaV2"/>
      <xsl:attribute name="deltaV2Complement" select="deltaxml:deltaV2ComplementValue(.)"/>

      <xsl:apply-templates select="deltaxml:text">
        <xsl:with-param name="textGroup-id" select="$textGroup-id"/>
        <xsl:with-param name="editType" select="$editType"/>
      </xsl:apply-templates>
    </dxml-merge-group:text>

  </xsl:template>
  
  <xsl:template match="deltaxml:textGroup/deltaxml:text">
    <xsl:param name="textGroup-id" as="xs:string"/>
    <xsl:param name="editType" as="xs:string"/>
    
    <xsl:variable name="isChosen" select="@deltaxml:chosen" as="xs:boolean"/>

    <dxml-merge:text>
      <xsl:attribute name="group" select="$textGroup-id"/>
      <xsl:attribute name="chosen" select="string($isChosen)"/>
      <xsl:attribute name="deltaV2" select="@deltaxml:deltaV2"/>

      <xsl:value-of select="text()" />
    </dxml-merge:text>
  </xsl:template>

  <xsl:template match="/*//*[not(ancestor-or-self::deltaxml:*)][@deltaxml:edit-type]">
    <xsl:variable name="element-id" select="generate-id()" as="xs:string"/>
    <xsl:variable name="editType" as="xs:string" select="@deltaxml:edit-type"/>
    <xsl:variable name="isChosen" select="@deltaxml:chosen" as="xs:boolean"/>
        
    <dxml-merge-group:element>
      <xsl:attribute name="group" select="$element-id"/>
      <xsl:attribute name="edit-type" select="$editType"/>
      <xsl:attribute name="deltaV2" select="@deltaxml:deltaV2"/>
      <xsl:attribute name="deltaV2Complement" select="deltaxml:deltaV2ComplementValue(.)"/>

      <dxml-merge:element group="d20e2" deltaV2="base=anna" chosen="false">
        <xsl:attribute name="group" select="$element-id"/>
        <xsl:attribute name="deltaV2" select="@deltaxml:deltaV2"/>
        <xsl:attribute name="chosen" select="$isChosen"/>
        <xsl:copy>
          <xsl:apply-templates select="(@* except @deltaxml:*), node()"/>
        </xsl:copy>
      </dxml-merge:element>
    </dxml-merge-group:element>
  </xsl:template>
  
  <xsl:template match="deltaxml:attributes">
    <xsl:apply-templates select="*" mode="output-attributes"/>
    <xsl:apply-templates select="*" mode="output-attribute-markup"/>
  </xsl:template>

  <xsl:template match="deltaxml:attributes/*" mode="output-attributes">
    <xsl:variable name="baseVersion" select="deltaxml:getBase(.)" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="@deltaxml:edit-type eq 'add'">
        <!-- Select the added version, if this is an add -->
        <xsl:attribute name="{local-name()}"
                       namespace="{deltaxml:convert-attribute-namespace(namespace-uri(.))}"
                       select="deltaxml:attributeValue/text()"/>
      </xsl:when>
      <xsl:when test="@deltaxml:edit-type eq 'delete'">
        <!-- No attribute to output when deleted -->
      </xsl:when>
      <xsl:when test="deltaxml:attributeValue[deltaxml:deltaV2Includes(@deltaxml:deltaV2, $baseVersion)]">
        <!-- Select the base version if there is one -->
        <xsl:attribute name="{local-name()}"
                       namespace="{deltaxml:convert-attribute-namespace(namespace-uri(.))}"
                       select="deltaxml:attributeValue[deltaxml:deltaV2Includes(@deltaxml:deltaV2, $baseVersion)]/text()"/>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="deltaxml:attributes/*" mode="output-attribute-markup">
    <xsl:variable name="attribute-id" select="generate-id()" as="xs:string"/>
    <xsl:if test="@deltaxml:edit-type">
      <xsl:variable name="mockAttribute" as="attribute()">
        <xsl:attribute name="{local-name()}"
                       namespace="{deltaxml:convert-attribute-namespace(namespace-uri(.))}"
                       select="''"/>
      </xsl:variable>
  
      <dxml-merge-group:attribute>
        <xsl:attribute name="group" select="$attribute-id"/>
        <xsl:attribute name="edit-type" select="@deltaxml:edit-type"/>
        <xsl:attribute name="name" select="$mockAttribute/name()"/>
        <xsl:attribute name="deltaV2" select="@deltaxml:deltaV2"/>
        <xsl:attribute name="deltaV2Complement" select="deltaxml:deltaV2ComplementValue(.)"/>

        <xsl:apply-templates select="deltaxml:attributeValue">
          <xsl:with-param name="attribute-id" select="$attribute-id"/>
          <xsl:with-param name="editType" select="@deltaxml:edit-type"/>
        </xsl:apply-templates>
      </dxml-merge-group:attribute>
    </xsl:if>
  </xsl:template>

  <xsl:template match="deltaxml:attributes/*/deltaxml:attributeValue">
    <xsl:param name="attribute-id" as="xs:string"/>
    <xsl:param name="editType" as="xs:string"/>
    <xsl:variable name="isChosen" select="@deltaxml:chosen" as="xs:boolean"/>
    <dxml-merge:attribute>
      <xsl:attribute name="group" select="$attribute-id"/>
      <xsl:attribute name="deltaV2" select="@deltaxml:deltaV2"/>
      <xsl:attribute name="chosen" select="$isChosen"/>
      <xsl:value-of select="text()"/>
    </dxml-merge:attribute>
  </xsl:template>
  
  
  
  <xsl:function name="deltaxml:deltaV2ComplementValue" as="xs:string">
    <xsl:param name="currentElement" as="node()"/>
    <xsl:variable name="childVersions" as="xs:string+" select="deltaxml:getVersions($currentElement/@deltaxml:deltaV2)" />
    <xsl:variable name="parentVersions" as="xs:string+" select="deltaxml:getParentVersions($currentElement)" />
    <xsl:value-of select="string-join(deltaxml:value-except($parentVersions, $childVersions), '=')"/>
  </xsl:function>
  
  <!-- 
    Copied from dx2-extract-version-moded.xsl file 1 April 2014
    
    In the dx2 format attributes are stored as elements and these elements have the same namespace as the corresponding attribute,
    with 2 exceptions, when the attribute wasn't in a namespace or was in the xml: namespace for example xml:space or xml:id.
    This function maps the dx2 format namespace into the original namespaces.
  -->
  <xsl:function name="deltaxml:convert-attribute-namespace" as="xs:string">
    <xsl:param name="namespace-uri" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="$namespace-uri eq 'http://www.deltaxml.com/ns/non-namespaced-attribute'"><xsl:text/></xsl:when>
      <xsl:when test="$namespace-uri eq 'http://www.deltaxml.com/ns/xml-namespaced-attribute'">http://www.w3.org/XML/1998/namespace</xsl:when>
      <xsl:otherwise><xsl:value-of select="$namespace-uri"/></xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
</xsl:stylesheet>
