<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:deltaxml="http://www.deltaxml.com/ns/well-formed-delta-v1"
  exclude-result-prefixes="xs"
  version="2.0">

  <xsl:function name="deltaxml:getBase" as="xs:string">
    <xsl:param name="inElement" as="element()"/>
    <xsl:sequence select="deltaxml:getVersions($inElement/ancestor-or-self::*[last()]
                                               /@deltaxml:deltaV2)[1]"/>
  </xsl:function>
  
  <xsl:function name="deltaxml:getVersions" as="xs:string+">
    <xsl:param name="deltaV2" as="xs:string"/>
    <xsl:sequence select="tokenize($deltaV2, '!=|=')"/>
  </xsl:function>

  <xsl:function name="deltaxml:deltaV2Includes" as="xs:boolean">
    <xsl:param name="deltaV2" as="xs:string"/>
    <xsl:param name="versionLabel" as="xs:string"/>
    <xsl:sequence select="deltaxml:getVersions($deltaV2) = $versionLabel"/>
  </xsl:function>

  <xsl:function name="deltaxml:getParentVersions" as="xs:string+">
    <xsl:param name="currentElement" as="node()"/>
    <xsl:sequence select="deltaxml:getVersions($currentElement/ancestor::*[@deltaxml:deltaV2][not(self::deltaxml:attributes)][1]/@deltaxml:deltaV2)"/>
  </xsl:function>

    <xsl:function name="deltaxml:value-intersect" as="xs:anyAtomicType*">
        <xsl:param name="arg1" as="xs:anyAtomicType*"/>
        <xsl:param name="arg2" as="xs:anyAtomicType*"/>
        <xsl:sequence select="distinct-values($arg1[.=$arg2])"/>
    </xsl:function>
    
    <xsl:function name="deltaxml:value-except" as="xs:anyAtomicType*">
        <xsl:param name="arg1" as="xs:anyAtomicType*"/>
        <xsl:param name="arg2" as="xs:anyAtomicType*"/>
        <xsl:sequence select="distinct-values($arg1[not(.=$arg2)])"/>
    </xsl:function>
    
  <!-- true when all members of arg2 are in arg1 -->
  <xsl:function name="deltaxml:subset" as="xs:boolean">
    <xsl:param name="arg1" as="xs:anyAtomicType*"/>
    <xsl:param name="arg2" as="xs:anyAtomicType*"/>
    <xsl:sequence select="empty(deltaxml:value-except($arg2, $arg1))"/>
  </xsl:function>


</xsl:stylesheet>