<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:dxml-merge="http://www.deltaxml.com/ns/dxml-merge"
  xmlns:dxml-merge-start="http://www.deltaxml.com/ns/dxml-merge-start"
  xmlns:dxml-merge-end="http://www.deltaxml.com/ns/dxml-merge-end"
  xmlns:dxml-merge-group="http://www.deltaxml.com/ns/dxml-merge-group"
  xmlns:dxml-merge-pi="http://www.deltaxml.com/ns/dxml-merge-pi"
  version="2.0">

  <xsl:output omit-xml-declaration="yes" method="xml"  />
  
  <xsl:template match="@* | node()[not(self::processing-instruction())]">
    <xsl:copy>
      <xsl:apply-templates select="@*, node()" />
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="processing-instruction('dxml-merge-start') | processing-instruction('dxml-merge-end') | processing-instruction('dxml-merge')">
    <xsl:variable name="piText" select="string(.)"/>
    <xsl:element name="{name(.)}:{dxml-merge:pseudo-name($piText)}">
      <xsl:call-template name="extract-pseudo-attributes">
        <xsl:with-param name="attribs" select="dxml-merge:pseudo-attribs($piText)"/>
      </xsl:call-template>
      <xsl:apply-templates select="dxml-merge:pseudo-content($piText)"/>
    </xsl:element>  
  </xsl:template>
  
  <xsl:template match="processing-instruction()">
    <xsl:element name="dxml-merge-pi:{name(.)}">
      <xsl:value-of select="string(.)"/>
    </xsl:element>  
  </xsl:template>  
  
  <xsl:template name="extract-pseudo-attributes">
    <xsl:param name="attribs" as="xs:string"/>
    <xsl:analyze-string select="$attribs" regex="^\s*([\w-]+)=&quot;([^&quot;]*)&quot;(.*)">
      <xsl:matching-substring>
        <xsl:attribute name="{regex-group(1)}" select="regex-group(2)" />
        
        <xsl:if test="regex-group(3) ne ''">
          <xsl:call-template name="extract-pseudo-attributes">
            <xsl:with-param name="attribs" select="regex-group(3)" />
          </xsl:call-template>
        </xsl:if>
      </xsl:matching-substring>
    </xsl:analyze-string>    
  </xsl:template>
  
  <xsl:function name="dxml-merge:pseudo-name" as="xs:string?"> 
    <xsl:param name="pi" as="xs:string" />
    <xsl:analyze-string select="$pi" regex="^\s*([\w-]+)\s">
      <xsl:matching-substring>
        <xsl:sequence select="regex-group(1)" />
      </xsl:matching-substring>
      <xsl:fallback>
        <xsl:sequence select="()" />
      </xsl:fallback>
    </xsl:analyze-string>
  </xsl:function>
  
  <xsl:function name="dxml-merge:pseudo-attribs" as="xs:string?"> 
    <xsl:param name="pi" as="xs:string" />
    <xsl:analyze-string select="$pi" regex="^\s*([\w-]+\s+)?([^&lt;]+)">
      <xsl:matching-substring>
        <xsl:sequence select="regex-group(2)"/> 
      </xsl:matching-substring>
    </xsl:analyze-string>
    <xsl:fallback>
      <xsl:sequence select="()" />
    </xsl:fallback>
  </xsl:function>
  
  <xsl:function name="dxml-merge:pseudo-content" as="node()*"> 
    <xsl:param name="pi" as="xs:string" />
    <xsl:analyze-string select="$pi" regex="^[^&lt;]*&lt;(.*)">
      <xsl:matching-substring>
        <xsl:variable name="fragment" select="saxon:parse(concat('&lt;fragment>', regex-group(1), '&lt;/fragment>'))" xmlns:saxon="http://saxon.sf.net/"/>
        <xsl:sequence select="$fragment/fragment/node()" />
      </xsl:matching-substring>
    </xsl:analyze-string>
    <xsl:fallback>
      <xsl:sequence select="()" />
    </xsl:fallback>
  </xsl:function>
    
</xsl:stylesheet>