<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:dxml-merge="http://www.deltaxml.com/ns/dxml-merge"
  xmlns:dxml-merge-start="http://www.deltaxml.com/ns/dxml-merge-start"
  xmlns:dxml-merge-end="http://www.deltaxml.com/ns/dxml-merge-end"
  xmlns:dxml-merge-group="http://www.deltaxml.com/ns/dxml-merge-group"
  xmlns:dxml-merge-pi="http://www.deltaxml.com/ns/dxml-merge-pi"
  version="2.0" exclude-result-prefixes="#all">

  <xsl:output omit-xml-declaration="yes" method="xml"  />
  
  <xsl:template match="@* | node()" mode="#all">
    <xsl:copy copy-namespaces="no">
      <xsl:apply-templates select="@*, node()" mode="#current" />
    </xsl:copy>
  </xsl:template>

  <xsl:template match="*[dxml-merge-start:*]">
    <xsl:copy copy-namespaces="no">
      <xsl:apply-templates select="@*" />
      <xsl:call-template name="group-potential-cluster">
        <xsl:with-param name="potential-cluster" select="node()" />
      </xsl:call-template>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="dxml-merge-start:* | dxml-merge-end:*" />
 
  <!-- 
    Part of a mutually recursive set of templates for processing a nested hierarchy of start and end group markers.
    Here each level of nesting is analysed and processed in turn.
    Note that the group item level processing is handled by another named template (the other template in the mutually recursive group).
  -->
  <xsl:template name="group-potential-cluster">
    <xsl:param name="potential-cluster" as="node()*" />
    
    <xsl:variable name="starts-and-ends" as="node()*" select="$potential-cluster[self::dxml-merge-start:* or self::dxml-merge-end:*]" />
    <xsl:variable name="top-level-group-ids" select="dxml-merge:top-level-group-ids($starts-and-ends, ())" />

    <!-- Find the start of each top-level group -->
    <xsl:for-each-group select="$potential-cluster" group-starting-with="dxml-merge-start:*[@group = ($top-level-group-ids)]">
      <xsl:variable name="start-node" select="current-group()[1]" as="node()"/>
       
      <xsl:choose>
        
        <!-- When we are at the start of a top-level group -->
        <xsl:when test="$start-node[self::dxml-merge-start:*[@group = $top-level-group-ids]]">
          <xsl:variable name="group-id" select="$start-node/@group" as="xs:string"/>

          <xsl:for-each-group select="current-group()" group-ending-with="dxml-merge-end:*[@group = $group-id]">
            <xsl:variable name="end-node" select="current-group()[last()]" as="node()" />
            <xsl:choose>

              <!-- When we are between a top-level start and end group marker -->
              <xsl:when test="$end-node[self::dxml-merge-end:*[@group = $group-id]]">
                <xsl:element name="dxml-merge-group:{local-name($start-node)}">
                  <xsl:apply-templates select="$start-node/@*" />
                  <xsl:choose>
                    <xsl:when test="local-name($start-node) eq 'attribute'">
                      <xsl:call-template name="process-group-items">
                        <xsl:with-param name="items" select="current-group()[self::dxml-merge:*[@group = $group-id]]"/>
                        <xsl:with-param name="contents" select="()" />                    
                      </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:call-template name="process-group-items">
                        <xsl:with-param name="items" 
                          select="current-group()[self::dxml-merge:*[@group = $group-id]]"/>
                        <xsl:with-param name="contents"
                          select="current-group()[not(self::dxml-merge:*[@group = $group-id] or 
                          self::dxml-merge-start:*[@group = $group-id] or 
                          self::dxml-merge-end:*[@group = $group-id])]"
                        />                    
                      </xsl:call-template>
                    </xsl:otherwise>
                  </xsl:choose>
               </xsl:element>
              </xsl:when>
              
              <!-- When we are not between a top-level start and end group marker -->
              <xsl:otherwise>
                <xsl:apply-templates select="current-group()" />
              </xsl:otherwise>
              
            </xsl:choose>
          </xsl:for-each-group>
        </xsl:when>
        
        <!-- When we are not at the start of a top level group -->
        <xsl:otherwise>
          <xsl:apply-templates select="current-group()" />
        </xsl:otherwise>

      </xsl:choose>
      
    </xsl:for-each-group>
    
  </xsl:template>
 
  <!--
    Each group may contain a number of dxml-merge:* elements (items) that are associated with it. 
    Process each of these items in turn.
    Note that at most one of these may be chosen.
    The chosen item contains all the non grouped elements (which may include other dxml:merge elements that have a different group id.
  -->
  <xsl:template name="process-group-items">
    <xsl:param name="items" as="node()*" />
    <xsl:param name="contents" as="node()*" />
    
    <xsl:for-each select="$items">
      <xsl:variable name="item" select="current()"/>
      <xsl:choose>
        <xsl:when test="$item[@chosen='false']">
          <xsl:apply-templates select="$item"/>
        </xsl:when>
        <xsl:when test="$item[@chosen='true']">
          <xsl:copy copy-namespaces="no">
            <xsl:apply-templates select="@*"/>
            
            <!-- Mutually recursive call to process any nested groups -->
            <xsl:call-template name="group-potential-cluster">
              <xsl:with-param name="potential-cluster" select="$contents" />
            </xsl:call-template>
          </xsl:copy>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message select="'Warning: impossible state ignored.'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>  
  </xsl:template>
 
  <!--
    Get the top level group identifiers for the given list of nodes, and current stack of nested groups.
  -->
  <xsl:function name="dxml-merge:top-level-group-ids">
    <xsl:param name="nodes" as="node()*" />
    <xsl:param name="stack" as="node()*" />
    <xsl:choose>
      <xsl:when test="count($nodes) = 0">
        <xsl:sequence select="()" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="head" select="$nodes[1]" as="node()" />
        <xsl:variable name="tail" select="remove($nodes, 1)" as="node()*" />
        <xsl:variable name="index" select="count($stack)" as="xs:integer"/>
        
        <xsl:choose>
          <xsl:when test="$head[self::dxml-merge-start:*]">
            <xsl:choose>
              <xsl:when test="$index = 0">
                <xsl:sequence select="$head/@group, dxml-merge:top-level-group-ids($tail, $head/@group)" />
              </xsl:when>
              <xsl:otherwise>
                <xsl:sequence select="dxml-merge:top-level-group-ids($tail, ($stack, $head/@group))" />
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:when test="$head[self::dxml-merge-end:*]">
            <xsl:choose>
              <xsl:when test="$index = 0">
                <xsl:message select="concat('Warning: Ignoring the ending a non existant merge start tag with end tag ', $head/@group)" />
                <xsl:sequence select="dxml-merge:top-level-group-ids($tail, $stack)" />
              </xsl:when>
              <xsl:when test="$stack[$index] = $head/@group">
                <xsl:sequence select="dxml-merge:top-level-group-ids($tail, remove($stack, $index))" />
              </xsl:when>
              <xsl:otherwise>
                <xsl:message select="concat('Warning: Ignoring unexpected tag end ', $head/@group, ' expected ', $stack[$index])" />
                <xsl:sequence select="dxml-merge:top-level-group-ids($tail, $stack)" />
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <!-- This should never happen - it is within the control of this xsl script -->
            <xsl:message select="'Fatal Error: invalid start and end merge list'" terminate="yes" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:function>
 
</xsl:stylesheet>