<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:dxml-merge="http://www.deltaxml.com/ns/dxml-merge"
  xmlns:dxml-merge-group="http://www.deltaxml.com/ns/dxml-merge-group"
  xmlns:dxml-merge-pi="http://www.deltaxml.com/ns/dxml-merge-pi"
  version="2.0" exclude-result-prefixes="#all">

  <xsl:output omit-xml-declaration="yes" method="xml"  />
  
  <!-- To serialize with saxon:serialize() -->
  <xsl:output name="default" omit-xml-declaration="yes" method="xml"  />
  
  <xsl:template match="@* | node()">
    <xsl:copy>
      <xsl:apply-templates select="@*, node()" mode="#current" />
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="node()" mode="nested">
    <xsl:variable name="tmp"><xsl:copy-of select="." copy-namespaces="no" /></xsl:variable>
    <xsl:value-of select="saxon:serialize($tmp, 'default')" xmlns:saxon="http://saxon.sf.net/"/>
  </xsl:template>
  
  <xsl:template match="dxml-merge-group:*">
    <xsl:variable name="pseudo-atts" select="for $att in (@*) return dxml-merge:pseudo-attribute(local-name($att), $att)" as="xs:string*"/>
    <xsl:variable name="groupId" select="@group" />
    
    <xsl:processing-instruction name="dxml-merge-start">
      <xsl:value-of select="local-name()"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="string-join($pseudo-atts, ' ')" />
    </xsl:processing-instruction>

    <xsl:apply-templates select="node()" />

    <xsl:processing-instruction name="dxml-merge-end">
      <xsl:value-of select="local-name()"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="dxml-merge:pseudo-attribute('group', $groupId)" />
    </xsl:processing-instruction>
  </xsl:template>
  
  <xsl:template match="dxml-merge:*">
    <xsl:variable name="pseudo-atts" select="for $att in (@*) return dxml-merge:pseudo-attribute(local-name($att), $att)" as="xs:string*"/>    
    <xsl:variable name="is-chosen" select="@chosen='true'" as="xs:boolean" />
    
    <xsl:processing-instruction name="dxml-merge">
      <xsl:value-of select="local-name()"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="string-join($pseudo-atts, ' ')" />
      <xsl:if test="not($is-chosen) or local-name() eq 'attribute'">
        <xsl:text> &lt;</xsl:text>
        <xsl:apply-templates mode="nested" />
      </xsl:if>
    </xsl:processing-instruction>
    
    <xsl:if test="$is-chosen and local-name() ne 'attribute'">
      <xsl:apply-templates select="node()" />
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="dxml-merge-pi:*">
    <xsl:processing-instruction name="{local-name()}">
      <xsl:value-of select="string(.)" />
    </xsl:processing-instruction>
  </xsl:template>
  
  <xsl:function name="dxml-merge:pseudo-attribute" as="xs:string"> 
    <xsl:param name="name" as="xs:string" />
    <xsl:param name="value" as="xs:string" />
    <xsl:value-of select="concat($name,'=&quot;', $value, '&quot;')" />
  </xsl:function>
  
</xsl:stylesheet>